@@ -1,4 +1,4 @@
 #!/bin/bash
 set -ev
 brew update
-brew install gcc@6
+brew install gcc@6 openssl
